@@ -12,6 +12,7 @@ from inspect import ismethod
 from inspect import ismethoddescriptor
 from inspect import ismodule
 from inspect import isroutine
+from functools import partial
 from logging import getLogger
 
 from .utils import basestring
@@ -35,6 +36,7 @@ except ImportError:
 
 
 __all__ = 'weave', 'Aspect', 'Proceed', 'Return', 'ALL_METHODS', 'NORMAL_METHODS'
+__version__ = '1.1.0'
 
 logger = getLogger(__name__)
 logdebug = logf(logger.debug)
@@ -70,7 +72,7 @@ class UnsupportedType(TypeError):
 
 class Proceed(object):
     """
-    Instructs the Aspect Calls to call the decorated function. Can be used multiple times.
+    Instruction for calling the decorated function. Can be used multiple times.
 
     If not used as an instance then the default args and kwargs are used.
     """
@@ -83,7 +85,9 @@ class Proceed(object):
 
 class Return(object):
     """
-    Instructs the Aspect to return a value.
+    Instruction for returning a *optional* value.
+
+    If not used as an instance then ``None`` is returned.
     """
     __slots__ = 'value',
 
@@ -95,22 +99,71 @@ class Aspect(object):
     """
     Container for the advice yielding generator. Can be used as a decorator on other function to change behavior
     according to the advices yielded from the generator.
-    """
-    __slots__ = 'advising_function'
 
-    def __init__(self, advising_function):
+    Args:
+        advising_function (generator function): A generator function that yields :ref:`advices`.
+        bind (bool): A convenience flag so you can access the cutpoint function (you'll get it as an argument).
+
+    Usage::
+
+        >>> @Aspect
+        ... def my_decorator(*args, **kwargs):
+        ...     print("Got called with args: %s kwargs: %s" % (args, kwargs))
+        ...     result = yield
+        ...     print(" ... and the result is: %s" % (result,))
+        >>> @my_decorator
+        ... def foo(a, b, c=1):
+        ...     print((a, b, c))
+        >>> foo(1, 2, c=3)
+        Got called with args: (1, 2) kwargs: {'c': 3}
+        (1, 2, 3)
+         ... and the result is: None
+
+    Normally you don't have access to the cutpoints (the functions you're going to use the aspect/decorator on) because
+    you don't and should not call them directly. There are situations where you'd want to get the name or other data
+    from the function. This is where you use the ``bind=True`` option::
+
+        >>> @Aspect(bind=True)
+        ... def my_decorator(cutpoint, *args, **kwargs):
+        ...     print("`%s` got called with args: %s kwargs: %s" % (cutpoint.__name__, args, kwargs))
+        ...     result = yield
+        ...     print(" ... and the result is: %s" % (result,))
+        >>> @my_decorator
+        ... def foo(a, b, c=1):
+        ...     print((a, b, c))
+        >>> foo(1, 2, c=3)
+        `foo` got called with args: (1, 2) kwargs: {'c': 3}
+        (1, 2, 3)
+         ... and the result is: None
+
+    """
+    __slots__ = 'advising_function', 'bind'
+
+    def __new__(cls, advising_function=UNSPECIFIED, bind=False):
+        if advising_function is UNSPECIFIED:
+            return partial(cls, bind=bind)
+        else:
+            self = super(Aspect, cls).__new__(cls)
+            self.__init__(advising_function, bind)
+            return self
+
+    def __init__(self, advising_function, bind=False):
         if not isgeneratorfunction(advising_function):
             raise ExpectedGeneratorFunction("advising_function %s must be a generator function." % advising_function)
         self.advising_function = advising_function
+        self.bind = bind
 
     def __call__(self, cutpoint_function):
         if isgeneratorfunction(cutpoint_function):
             if PY3:
                 from aspectlib.py3support import decorate_advising_generator_py3
-                return decorate_advising_generator_py3(self.advising_function, cutpoint_function)
+                return decorate_advising_generator_py3(self.advising_function, cutpoint_function, self.bind)
             else:
                 def advising_generator_wrapper(*args, **kwargs):
-                    advisor = self.advising_function(*args, **kwargs)
+                    if self.bind:
+                        advisor = self.advising_function(cutpoint_function, *args, **kwargs)
+                    else:
+                        advisor = self.advising_function(*args, **kwargs)
                     if not isgenerator(advisor):
                         raise ExpectedGenerator("advising_function %s did not return a generator." % self.advising_function)
                     try:
@@ -174,7 +227,10 @@ class Aspect(object):
                 return mimic(advising_generator_wrapper, cutpoint_function)
         else:
             def advising_function_wrapper(*args, **kwargs):
-                advisor = self.advising_function(*args, **kwargs)
+                if self.bind:
+                    advisor = self.advising_function(cutpoint_function, *args, **kwargs)
+                else:
+                    advisor = self.advising_function(*args, **kwargs)
                 if not isgenerator(advisor):
                     raise ExpectedGenerator("advising_function %s did not return a generator." % self.advising_function)
                 try:
@@ -438,6 +494,7 @@ def weave_instance(instance, aspect, methods=NORMAL_METHODS, lazy=False, **optio
                 )
     return entanglement
 
+
 def weave_module(module, aspect, methods=NORMAL_METHODS, lazy=False, **options):
     """
     Low-level weaver for "whole module weaving".
@@ -461,7 +518,7 @@ def weave_module(module, aspect, methods=NORMAL_METHODS, lazy=False, **options):
                     weave_class(func, aspect, owner=module, name=attr, methods=methods, lazy=lazy, **options),
                     #  it's not consistent with the other ways of weaving a class (it's never weaved as a routine).
                     #  therefore it's disabled until it's considered useful.
-                    #weave_module_function(module, getattr(module, attr), aspect, force_name=attr, **options),
+                    #  #weave_module_function(module, getattr(module, attr), aspect, force_name=attr, **options),
                 )
     return entanglement
 
@@ -483,7 +540,7 @@ def weave_class(klass, aspect, methods=NORMAL_METHODS, subclasses=True, lazy=Fal
             if not issubclass(sub_class, Fabric):
                 entanglement.merge(weave_class(sub_class, aspect, methods=methods, subclasses=subclasses, lazy=lazy))
     logdebug("weave_class (klass=%r, methods=%s, subclasses=%s, lazy=%s, owner=%s, name=%s, aliases=%s)",
-          klass, methods, subclasses, lazy, owner, name, aliases)
+             klass, methods, subclasses, lazy, owner, name, aliases)
     if lazy:
         def __init__(self, *args, **kwargs):
             super(SubClass, self).__init__(*args, **kwargs)
@@ -524,7 +581,7 @@ def weave_class(klass, aspect, methods=NORMAL_METHODS, subclasses=True, lazy=Fal
     return entanglement
 
 
-def patch_module(module, name, replacement, original=UNSPECIFIED, aliases=True, location=None, **bogus_options):
+def patch_module(module, name, replacement, original=UNSPECIFIED, aliases=True, location=None, **_bogus_options):
     """
     Low-level attribute patcher.
 
@@ -582,6 +639,6 @@ def weave_module_function(module, target, aspect, force_name=None, **options):
     :returns: An :obj:`aspectlib.Entanglement` object.
     """
     logdebug("weave_module_function (module=%s, target=%s, aspect=%s, force_name=%s, **options=%s",
-          module, target, aspect, force_name, options)
+             module, target, aspect, force_name, options)
     name = force_name or target.__name__
     return patch_module(module, name, _checked_apply(aspect, target, module=module), original=target, **options)
