@@ -1,7 +1,6 @@
 from __future__ import print_function
 
 import sys
-from functools import wraps
 from inspect import isgenerator
 from inspect import isgeneratorfunction
 from logging import getLogger
@@ -9,18 +8,20 @@ from logging import getLogger
 from aspectlib import ExpectedGenerator
 from aspectlib import mimic
 from aspectlib import Proceed
-from aspectlib import PY3
 from aspectlib import Return
 from aspectlib import UnacceptableAdvice
 
 logger = getLogger(__name__)
 
 
-def decorate_advising_generator_py3(advising_function, cutpoint_function):
+def decorate_advising_generator_py3(advising_function, cutpoint_function, bind):
     assert isgeneratorfunction(cutpoint_function)
 
     def advising_generator_wrapper_py3(*args, **kwargs):
-        advisor = advising_function(*args, **kwargs)
+        if bind:
+            advisor = advising_function(cutpoint_function, *args, **kwargs)
+        else:
+            advisor = advising_function(*args, **kwargs)
         if not isgenerator(advisor):
             raise ExpectedGenerator("advising_function %s did not return a generator." % advising_function)
         try:
@@ -34,7 +35,7 @@ def decorate_advising_generator_py3(advising_function, cutpoint_function):
                     gen = cutpoint_function(*args, **kwargs)
                     try:
                         result = yield from gen
-                    except BaseException as exc:
+                    except BaseException:
                         advice = advisor.throw(*sys.exc_info())
                     else:
                         try:
