@@ -1,3 +1,4 @@
+from sys import _getframe
 from collections import defaultdict
 from collections import namedtuple
 from difflib import unified_diff
@@ -6,8 +7,6 @@ from functools import wraps
 from inspect import isclass
 from logging import getLogger
 from traceback import format_stack
-import ast
-import sys
 
 from aspectlib import ALL_METHODS
 from aspectlib import mimic
@@ -15,6 +14,7 @@ from aspectlib import weave
 
 from .utils import camelcase_to_underscores
 from .utils import container
+from .utils import logf
 from .utils import qualname
 from .utils import repr_ex
 from .utils import Sentinel
@@ -23,10 +23,6 @@ try:
     from dummy_thread import allocate_lock
 except ImportError:
     from _dummy_thread import allocate_lock
-try:
-    from StringIO import StringIO
-except ImportError:
-    from io import StringIO
 try:
     from collections import OrderedDict
 except ImportError:
@@ -39,6 +35,7 @@ except ImportError:
 __all__ = 'mock', 'record', "Story"
 
 logger = getLogger(__name__)
+logexception = logf(logger.exception)
 
 Call = namedtuple('Call', ('self', 'args', 'kwargs'))
 CallEx = namedtuple('CallEx', ('self', 'name', 'args', 'kwargs'))
@@ -234,7 +231,8 @@ class StoryResultWrapper(object):
         '__contains__', '__call__', '__lt__', '__le__', '__ne__', '__gt__', '__ge__', '__cmp__', '__rcmp__',
         '__nonzero__',
     ):
-        exec ("%s = __unsupported__" % mm)
+        exec("%s = __unsupported__" % mm)
+
 
 class StoryFunctionWrapper(object):
     def __init__(self, wrapped, handle, binding=None, owner=None):
@@ -385,6 +383,11 @@ class Story(_RecordingBase):
     """
     FunctionWrapper = StoryFunctionWrapper
 
+    def __init__(self, *args, **kwargs):
+        super(Story, self).__init__(*args, **kwargs)
+        frame = _getframe(1)
+        self._context = frame.f_globals, frame.f_locals
+
     def replay(self, **options):
         """
         :param bool proxy:
@@ -415,7 +418,10 @@ class Story(_RecordingBase):
                  mymod.func('some arg') == 'some result'  # returns
                 -mymod.func('other arg') == 'other result'  # returns
                 +mymod.func('bogus arg') == None  # returns
-
+            ACTUAL:
+                mymod.func('some arg') == 'some result'  # returns
+                mymod.func('bogus arg') == None  # returns
+            <BLANKLINE>
         """
         options.update(self._options)
         return Replay(self, **options)
@@ -423,23 +429,12 @@ class Story(_RecordingBase):
 ReplayPair = namedtuple("ReplayPair", ('expected', 'actual'))
 
 
-def logged_eval(value):
+def logged_eval(value, context):
     try:
-        _node = ast.parse(value)
-        try:
-            _attr = _node.body[0].value.func
-            if isinstance(_attr, ast.Attribute):
-                _code = "import %s" % _attr.value.id
-                try:
-                    exec(_code)
-                except ImportError:
-                    logger.error("Failed to %r for %r", _code, value)
-        except AttributeError:
-            pass
-        return eval(value)
+        return eval(value, *context)
     except:
-        logger.exception("Failed to evaluate %r.\nContext:\n%s", value, ''.join(format_stack(
-            f=sys._getframe(1),
+        logexception("Failed to evaluate %r.\nContext:\n%s", value, ''.join(format_stack(
+            f=_getframe(1),
             limit=15
         )))
         raise
@@ -461,6 +456,7 @@ class Replay(_RecordingBase):
         self._proxy = proxy
         self._strict = strict
         self._dump = dump
+        self._context = play._context
         self._recurse_lock = allocate_lock() if recurse_lock is True else (recurse_lock and recurse_lock())
 
     def _handle(self, binding, name, args, kwargs, wrapped, bind=None):
@@ -470,11 +466,11 @@ class Replay(_RecordingBase):
             if isinstance(result, _Binds):
                 self._tag_result(name, bind)
             elif isinstance(result, _Returns):
-                return logged_eval(result.value)
+                return logged_eval(result.value, self._context)
             elif isinstance(result, _Raises):
-                raise logged_eval(result.value)
+                raise logged_eval(result.value, self._context)
             else:
-                raise RuntimeError('Internal failure - unknown resultt: %r' % result)  # pragma: no cover
+                raise RuntimeError('Internal failure - unknown result: %r' % result)  # pragma: no cover
         else:
             if self._proxy:
                 shouldrecord = not self._recurse_lock or self._recurse_lock.acquire(False)
@@ -497,7 +493,18 @@ class Replay(_RecordingBase):
             else:
                 raise AssertionError("Unexpected call to %s/%s with args:%s kwargs:%s" % pk)
 
-    def unexpected(self, _missing=False):
+    def _unexpected(self, _missing=False):
+        if _missing:
+            expected, actual = self._actual, self._expected
+        else:
+            actual, expected = self._actual, self._expected
+        return ''.join(_format_calls(OrderedDict(
+            (pk, val) for pk, val in actual.items()
+            if pk not in expected or val != expected.get(pk)
+        )))
+
+    @property
+    def unexpected(self):
         """
         Returns a pretty text representation of just the unexpected calls.
 
@@ -514,7 +521,7 @@ class Replay(_RecordingBase):
             ...         print(exc)
             Got some arg in the real code !
             boom!
-            >>> print(replay.unexpected())
+            >>> print(replay.unexpected)
             mymod.func('some arg') == None  # returns
             mymod.badfunc() ** ValueError('boom!',)  # raises
             <BLANKLINE>
@@ -534,21 +541,16 @@ class Replay(_RecordingBase):
             boom!
 
         """
-        if _missing:
-            expected, actual = self._actual, self._expected
-        else:
-            actual, expected = self._actual, self._expected
-        return ''.join(_format_calls(OrderedDict(
-            (pk, val) for pk, val in actual.items()
-            if pk not in expected or val != expected.get(pk)
-        )))
+        return self._unexpected()
 
+    @property
     def missing(self):
         """
         Returns a pretty text representation of just the missing calls.
         """
-        return self.unexpected(_missing=True)
+        return self._unexpected(_missing=True)
 
+    @property
     def diff(self):
         """
         Returns a pretty text representation of the unexpected and missing calls.
@@ -561,17 +563,28 @@ class Replay(_RecordingBase):
         expected = list(_format_calls(self._expected))
         return ''.join(unified_diff(expected, actual, fromfile='expected', tofile='actual'))
 
+    @property
+    def actual(self):
+        return ''.join(_format_calls(self._actual))
+
+    @property
+    def expected(self):
+        return ''.join(_format_calls(self._expected))
+
     def __exit__(self, *args):
         super(Replay, self).__exit__()
         if self._strict or self._dump:
-            diff = self.diff()
+            diff = self.diff
             if diff:
                 if self._dump:
                     print('STORY/REPLAY DIFF:')
                     print('    ' + '\n    '.join(diff.splitlines()))
+                    print('ACTUAL:')
+                    print('    ' + '    '.join(_format_calls(self._actual)))
                 if self._strict:
                     raise AssertionError(diff)
 
+
 def _format_calls(calls):
     for (binding, name, args, kwargs), result in calls.items():
         sig = '%s(%s%s%s)' % (name, args, ', ' if kwargs and args else '', kwargs)
