@@ -12,6 +12,7 @@ from process_tests import wait_for_strings
 import aspectlib
 from aspectlib.test import mock
 from aspectlib.test import record
+from aspectlib.utils import PYPY
 
 try:
     import thread
@@ -92,7 +93,7 @@ def test_socket_as_string_target():
 
 def test_socket_meth(meth=socket.socket.close):
     calls = []
-    with aspectlib.weave(meth, record(history=calls)):
+    with aspectlib.weave(meth, record(calls=calls)):
         s = socket.socket()
         assert s.close() is None
     assert calls == [(s, (), {})]
@@ -120,7 +121,7 @@ def test_socket_all_methods():
     assert "}.__init__ => None" in buf.getvalue()
 
 
-@pytest.mark.skipif(not hasattr(os, 'fork') or aspectlib.PYPY, reason="os.fork not available or PYPY")
+@pytest.mark.skipif(not hasattr(os, 'fork') or PYPY, reason="os.fork not available or PYPY")
 def test_realsocket_makefile():
     buf = StringIO()
     p = socket.socket()
@@ -171,3 +172,16 @@ def test_realsocket_makefile():
                 f.write('-\n')
         finally:
             os._exit(0)
+
+
+def test_weave_os_module():
+    calls = []
+
+    with aspectlib.weave('os', record(calls=calls, extended=True), methods="getenv|walk"):
+        os.getenv('BUBU', 'bubu')
+        os.walk('.')
+
+    assert calls == [
+        (None, 'os.getenv', ('BUBU', 'bubu'), {}),
+        (None, 'os.walk', ('.',), {})
+    ]
